function peakDict = BOP_buildingDictionary(peaksPpm,DictSize,DictRange,verbose)
% peakDict = BOP_buildingDictionary(peaksPpm,DictSize,DictRange,verbose)
% 
% Bag of Peaks: create Dictionary via clustering
% 
% peaksPpm: Dxtopn matrix, containing the locations of the topn peaks
%     extracted from all the D peaks -- as result of the BOP_extractPeaks
%     function
% DictSize: desired size of dictionary. if DictSize == -1, 
%     then the size is automatically estimated via the Davies Bouldin index
% DictRange: vector containing the possible dictionary size
%     (to be investigated in the automatic search)
% verbose: display some information
%
% peakDict: vector containing the location of the peaks of the Dictionary



data = peaksPpm(:);
Y = pdist(data,'euclid');
if verbose, fprintf('Clustering peaks ... '), end;
Z = linkage(Y,'complete');
if verbose, fprintf('done\n'), end;
if DictSize == -1
    % automatic selection of the size via the Davise-Bouldin Index
    if verbose, fprintf('Building Dictionary (automatic size)\n'), end;
    for nW = DictRange
        lab = cluster(Z,'maxclust',nW);
        DBlab{nW} = lab;
    end
    % computing DB_index
    for nW = DictRange
        DB(nW) = DBIndex(peaksPpm(:),DBlab{nW});
    end
    if verbose, figure, plot(DictRange,DB(DictRange)), xlabel('Dictionary Size'), ylabel('DB index'), end
    [~,cc] = min(DB(DictRange));
    bestW = DictRange(cc);
    lab = DBlab{bestW};
    nclus = length(unique(lab));
    if verbose, fprintf('Automatic Size %d\n',nclus), end
    peakDict = zeros(nclus,1);
    for k = 1:nclus
        l = data(lab==k,:);
        peakDict(k) = mean(l);
    end
else
    nW = DictSize;
    % nW given
    % complete link (deterministic)
    if verbose, fprintf('Building Dictionary: %d peaks\n',nW), end;
    lab = cluster(Z,'maxclust',nW);
    nclus = length(unique(lab));
    peakDict = zeros(nW,1);
    for k = 1:nclus
        l = data(lab==k,:);
        peakDict(k) = mean(l);
    end
end


function DB = DBIndex(data,lab)
% DB = DBIndex(data,lab)
%
% Davies Bouldin Index computed as in Jain Book 1988
%

[~,ndim] = size(data);
nclus = length(unique(lab));
e = zeros(1,nclus);
mu = zeros(nclus,ndim);
d = zeros(nclus,nclus);
% e
for k = 1:nclus
    ll = data((lab == k),:);
    dims(k) = size(ll,1);
    mu(k,:) = mean(ll);
    temp1 = (ll-repmat(mu(k,:),[dims(k),1])).^2;
    temp2 = sum(temp1,2);
    e(k) = mean(sqrt(temp2));
end

% d
temp1 = kron(mu,ones(nclus,1));
temp2 = repmat(mu,[nclus,1]);
temp3 = (temp1-temp2).^2;
temp4 = sqrt(sum(temp3,2));
d = reshape(temp4,[nclus nclus]);

% Rjk
R = repmat(e,[nclus 1]) + repmat(e',[1 nclus]);
R = R./d; %Nan are overwritten in the next stage

% Rk
R = R - diag(inf(1,nclus));
Rk = max(R);

% DB
DB = mean(Rk);

